import logging
import os

from print_format import *
import ACCDIST.build_models as build_models
from ACCDIST import analysis

def main(model, filepath, name, do_absolute_similarity, dodist, doACCDIST, doMeanModel, docorrelationtables, doPhonemes, doNearestNeighbour, doquiet, minSegmentCount=50):
    f = open(os.path.join(filepath,'output-{}.log'.format(name)), 'w')
    f.write("found %3d different accent names\n" % len(model.accents))
    # print this_analysis.accents
    f.write("found %3d different gender names\n" % len(model.genders))
    # print this_analysis.genders
    f.write("found %3d different speaker names\n" % len(model.speakers))
    # print [s.id for s in this_analysis.speakers]
    f.write("found %3d different segment names\n" % len(model.transcriptions))
    # print [s.transcription for s in this_analysis.segments]
    speakers_missing_segments = [s for s in model.speakers if len(model.get_speakers_segments(s)) < minSegmentCount]
    if len(speakers_missing_segments) > 0:
        logging.error("Found the following speakers with less than {:d} segments: {}\n".format(
                  minSegmentCount,
                  [str(s) for s in speakers_missing_segments]))
        #for s in speakers_missing_segments:
        #    model.speakers.remove(s)

    # /* collapse duplicate segments to one average entry */
    # segments.collapse()

    if doMeanModel:
        meanDistanceTable = build_models.MeanDistanceTable(model=model)
        for accent in model.accents:
            print("Calculating mean vowels for accent {}".format(accent))
            accent_file = open(os.path.join(filepath, 'output-mean_vowel_distance-{}.log'.format(accent)), 'w',encoding="utf-8")
            #print_phylip_table(meanDistanceTable.segment_table[accent].table, accent_file)
            print_list(meanDistanceTable.accent_table[accent].table, accent_file)

    # /* calculate absolute similarity score */
    if do_absolute_similarity:
        if doMeanModel:
            # /*build  mean models for accents first */
            f.write("Absolute similarity - mean\n")
            build_models.absolute_distance_with_mean(model, meanDistanceTable)
            print_table(model.absolute_mean_closest_table, "Closest mean accent, using absolute distance", False, f)
            print_table(model.absolute_mean_all_table, "Distance to mean accent, using absolute distance", True, f)
            f.write("Absolute similarity - mean excluding self")
            build_models.absolute_distance_with_mean_excluding_self(model, meanDistanceTable)
            print_table(model.absolute_selfexcluding_mean_closest,
                        "Closest mean accent, using absolute distance, excluding self", False, f)
            print_table(model.absolute_selfexcluding_mean_all_table,
                        "Distance to mean accent, using absolute distance, excluding self", True, f)


        if docorrelationtables:
            f.write("Absolute correlation tables\n")
            absolute_speaker_distance_table = dict()
            for speaker1 in model.speakers_with_segments(minSegmentCount):
                absolute_speaker_distance_table[speaker1.id] = dict()
                for speaker2 in model.speakers_with_segments(minSegmentCount):
                    c = build_models.compute_absolute_table_distance(speaker1.segments,speaker2.segments)
                    model.absolute_all_table[speaker1.id][speaker2.id] = c
            print_table(model.absolute_all_table, "Absolute distance for each speaker pair", True, f)


        if doNearestNeighbour:  # one nearest neighbour
            f.write("Absolute distance, nearest neighbour\n")
            correct = 0
            bad = 0
            for speaker in model.speakers_with_segments(minSegmentCount):
                correlation = 1E10
                closest_speaker = None
                for speaker2 in [s for s in model.speakers_with_segments(minSegmentCount) if s is not speaker]:
                    c = build_models.compute_absolute_table_distance(speaker.segments,
                                                                     speaker2.segments)
                    if c is not None and c < correlation:
                        correlation = c
                        closest_speaker = speaker2
                if doquiet <= 1:
                    f.write("Speaker \t%s\t is closest to speaker \t%s\t with a distance of \t%s\n" % (speaker.id, closest_speaker, correlation))
                if speaker.accent == closest_speaker.accent:
                    correct += 1
                else:
                    bad += 1
                model.absolute_nearest_neighbour[speaker.accent][closest_speaker.accent] += 1
            print_table(model.absolute_nearest_neighbour, "Nearest neighbour, using absolute distance", False, f)

    # /* calculate between-segment distance tables */
    if dodist:
        print("Calculating intersegment distance table")
        for speaker in model.speakers_with_segments(1):
            speaker.set_intersegment_distance_table()
            # if dodumpdist: #dodumpdist not yet defined as an input argument
            #     for segment in speaker.segments:
            #         for segment2 in speaker.segments:
            #             f.write("Speaker %s\t%s\t%s\t%s\n" % (speaker.id, segment, segment2, speaker.inter_segments_distance_table.get(segment, segment2)))


    # /* correlation distance */
    if doACCDIST:
        for speaker in model.speakers:
            speaker.set_intersegment_distance_table()

        if doMeanModel:
            print("Doing mean ACCDIST")
            f.write("Correlation similarity - mean\n")
            build_models.correlation_with_mean(model, meanDistanceTable.speaker_list())
            print_table(model.accdist_mean_closest_table, "Closest mean accent, using ACCDIST", False, f)
            print_table(model.accdist_mean_all_table, "Correlation with mean accent, using ACCDIST", True, f)
            build_models.correlation_with_mean_excluding_self(model, meanDistanceTable)
            print_table(model.accdist_selfexcluding_mean_closest, "Closest mean accent, excluding self, using ACCDIST", False, f)
            print_table(model.accdist_selfexcluding_mean_all_table, "Correlation with mean accent, excluding self, using ACCDIST", True, f)


        if docorrelationtables:
            print("Doing ACCDIST tables per speaker")
            f.write("Correlation tables for each speaker\n")
            for speaker in model.speakers:
                for speaker2 in [s for s in model.speakers if s is not speaker]:
                    correlation = build_models.compute_correlation_table_distance(speaker, speaker2)
                    model.accdist_all_table[speaker.id][speaker2.id] = correlation
            #for speaker in [key for key in model.accdist_all_table.keys() if not key in model.speakers]:
            #    del model.accdist_all_table[speaker]
            #for kvpairs in [d for d in model.accdist_all_table.values()]:
            #    for speaker2 in [s2 for s2 in kvpairs.keys() if not s2 in model.speakers]:
            #        del model.accdist_all_table[dict][speaker2]

            print_speaker_list(model.accdist_all_table, model.speakers, "ACCDIST for each speaker pair", f)
            print_table(model.accdist_all_table, "ACCDIST for each speaker pair", True, f)

        if doPhonemes:
            print("Doing phoneme tables")
            f.write("Phoneme tables\n")
            for speaker in model.speakers:
                print(speaker)
                table = speaker.intersegment_distance_table.m
                for s1 in table:
                    for s2 in table[s1]:
                        if not s1 == s2:
                            f.write("%s\t%s\t%s\t%s\t%s\t%s\n" % (speaker.id, speaker.gender, speaker.accent, s1, s2, table[s1][s2]))

        if doNearestNeighbour:  # one nearest neighbour
            f.write("ACCDIST, nearest neighbour\n")
            correct = 0
            bad = 0
            for speaker in model.speakers_with_segments(minSegmentCount):
                correlation = -1
                closest_speaker = None
                for speaker2 in [s for s in model.speakers_with_segments(minSegmentCount) if s is not speaker]:
                    c = build_models.compute_correlation_table_distance(speaker, speaker2)
                    if c is not None and c > correlation:
                        correlation = c
                        closest_speaker = speaker2
                if doquiet <= 1:
                    f.write("Speaker \t%s\t is closest to speaker \t%s\t with a correlation of \t%s\n" % (speaker.id, closest_speaker.id, correlation))
                if speaker.accent == closest_speaker.accent:
                    correct += 1
                else:
                    bad += 1
                model.accdist_nearest_neighbour[speaker.accent][closest_speaker.accent] += 1
            print_table(model.accdist_nearest_neighbour, "Nearest neighbour, using ACCDIST", False, f)
    f.close()



def get_accents_with_over_n_speakers(n, min_segment_count, model):
    a = [s.accent for s in model.speakers]
    d = {x: a.count(x) for x in a}
    relevant_accents = []
    for a in d:
        if d[a] > n:
            relevant_accents.append(a)
    new_data = analysis.Data()
    new_data.accents = relevant_accents
    new_data.speakers = [s for s in model.speakers if s.accent in relevant_accents]
    new_data.segments = [s for s in model.segments if s.speaker in new_data.speakers]
    new_data.genders = list(set([s.gender for s in model.speakers]))

    new_data.speakers = [s for s in model.speakers if len(new_data.get_speakers_segments(s)) > min_segment_count]

    return analysis.Analysis(new_data)


def get_transcriptions_except_t(t, model):
    new_data = analysis.Data()
    new_data.accents = model.accents
    new_data.speakers = model.speakers
    new_data.segments = [s for s in model.segments if s.transcription is not t]
    new_data.genders = list(set([s.gender for s in model.speakers]))
    return analysis.Analysis(new_data)