import Segment

class Data:
    def __init__(self):
        self.accents = []
        self.genders = []
        self.speakers = []
        self.segments = []
        self.data_length = None

    def add_accent(self, accent):
        if accent not in self.accents:
            self.accents.append(accent)

    #speaker object
    def add_speaker(self, speaker):
        if speaker not in self.speakers:
            self.speakers.append(speaker)

    def add_gender(self, gender):
        if gender not in self.genders:
            self.genders.append(gender)

    #segment object
    def add_segment(self, segment):
        if segment not in self.segments:
            self.segments.append(segment)

    def transcriptions(self):
        return set(s.transcription for s in self.segments)

    def get_speakers(self):
        return list(set(s.speaker for s in self.segments))

    def get_speakers_segments(self, speaker):
        return list(s for s in self.segments if s.speaker==speaker)

    def collapse_segments(self):
        average_segments = []
        for speaker in self.speakers:
            segments = self.get_speakers_segments(speaker)
            transcriptions = set([s.transcription for s in segments])
            sum_data = dict()
            count = dict()
            mean_data = dict()
            for transcription in transcriptions:
                sum_data[transcription] = [0.0 for i in range(len(segments[0].data))]
                count[transcription] = 0
                for segment in [s for s in segments if s.transcription == transcription]:
                    sum_data[transcription] = [sum(x) for x in zip(sum_data[segment.transcription], segment.data)]
                    count[transcription] += 1
                mean_data[transcription] = [x / count[transcription] for x in sum_data[transcription]]

            for key in mean_data:
                average_segments.append(Segment(speaker, key, mean_data[key]))

        return average_segments