# -*- coding: utf-8 -*-
import Data
import Segment
import Speaker


def data_table(table, rows, columns):
        for i in rows:
            table[i] = dict()
            for j in columns:
                table[i][j] = 0
        return table


class Analysis:
    def __init__(self, data):
        self.speakers = data.speakers
        self.accents = data.accents
        self.genders = data.genders
        self.segments = data.collapse_segments()
        self.transcriptions = data.transcriptions()

        speaker_ids = [s.id for s in self.speakers]
        self.absolute_mean_closest_table = data_table(dict(), self.accents, self.accents)
        self.absolute_selfexcluding_mean_closest = data_table(dict(), self.accents, self.accents)
        self.absolute_mean_all_table = data_table(dict(), speaker_ids, self.accents)
        self.absolute_selfexcluding_mean_all_table = data_table(dict(), speaker_ids, self.accents)
        self.absolute_all_table = data_table(dict(), speaker_ids, speaker_ids)
        self.absolute_nearest_neighbour = data_table(dict(), self.accents, self.accents)
        self.accdist_mean_closest_table = data_table(dict(), self.accents, self.accents)
        self.accdist_selfexcluding_mean_closest = data_table(dict(), self.accents, self.accents)
        self.accdist_mean_all_table = data_table(dict(), speaker_ids, self.accents)
        self.accdist_selfexcluding_mean_all_table = data_table(dict(), speaker_ids, self.accents)
        self.accdist_all_table = data_table(dict(), speaker_ids, speaker_ids)
        self.accdist_nearest_neighbour = data_table(dict(), self.accents, self.accents)
        try:
            self.data_length = len(data.segments[0].data)
        except IndexError:
            print("No data in ANALtoMFCC.lst")

    def get_speakers_segments(self, speaker):
        return list(s for s in self.segments if s.speaker==speaker)

    def get_segment(self, speaker, transcription):
        return (s for s in self.segments if s.speaker==speaker and s.transcription==transcription)

    def speakers_with_segments(self, number_of_segments):
        return list(s for s in self.speakers if len(self.get_speakers_segments(s)) > number_of_segments)


# /* analyse a table for accents and speakers */
def analyse_table(filename):
    upload = Data()

    with open(filename) as f:
        content = f.readlines()

    for line in content:
        parts = line.split('\t')
        accent = parts[0]
        gender = parts[1]
        speaker = parts[2]
        vowel = parts[3]
        data = parts[4:]

        if len(data) is not 26:
            raise Exception()
        else:
            data = data[0:11] + data[13:24]

        upload.add_accent(accent)
        upload.add_gender(gender)
        current_speaker = Speaker(speaker, accent, gender)
        upload.add_speaker(current_speaker)
        segment = Segment(current_speaker, vowel, data)
        upload.add_segment(segment)
        upload.add_segment(segment)

    for speaker in upload.speakers:
        speaker.set_segments(upload.get_speakers_segments(speaker))

    return Analysis(upload)