import getopt
import sys
import ACCDIST.analysis as analysis
from ACCDIST.ACCDIST import main, get_accents_with_over_n_speakers


def callACCDIST(opts, outputFilepath=".", outputType="", minSegmentCount=50):
    PROGNAME = "ACCDIST"
    PROGVERS = "1.3"
    do_absolute_similarity = 0
    dodist = 0
    do_ACCDIST_similarity = 0
    doquiet = 0
    doMeanModel = 0
    docorrelationtables = 0
    doPhonemes = 0
    doNearestNeighbour = 0
    dogroupbyaccent = 0
    error_flag = 0

    filename = "../../Results/anal2mfcc-phonemes.lst" #Default filename
    argument_count = len(opts)

    for option, argument in opts:
        if option == 'f':
            filename = argument
            print("Filename = {}".format(argument))
        elif option == 'I':
            print("{}: accent distance metric Vs{}\n".format(PROGNAME, PROGVERS))
            exit(0)
        elif option == 'a':		# /* absolute metric on raw data */
            do_absolute_similarity=1
            dodist=0
            do_ACCDIST_similarity=0
        elif option == 'c':		# /* correlation metric on distance tables */
            do_absolute_similarity=0
            dodist=1
            do_ACCDIST_similarity=1
        elif option == 'q':		# /* quiet mode */
            doquiet += 1
        elif option == 'm':		# /* recognise by comparison to mean model */
            doMeanModel=1
        elif option == 'D':		# /* dump raw distance tables */
            dodumpdist=1
        elif option == 'd':
            docorrelationtables=1		# /*dump correlation tables*/
        elif option == 'L':
            dogroupbyaccent = 1		# /*dump average accent correlation table*/
        else:
            raise Exception("usage: %s (-q) (-a|-c) (-m) (-D) (-d) *.txt\n" % PROGNAME)

    #for debugging
    # filename = "temp.lst" #subsection of data
    # Each phoneme independent
    # filename = "../../Results/anal2mfcc-full-data.lst" # Each word transcribed individually

    # Absolute distance (will show gender, age) and/or correlation distance (ACCDIST)
    #do_absolute_similarity = True
    #do_ACCDIST_similarity = True

    # Distance to mean, distance to all individuals, and/or nearest neighbour
    #doMeanModel = True
    docorrelationtables = True
    #doPhonemes = True
    doNearestNeighbour = True


    # ???
    #dodist = True
    #dodumpdist = True

    # /* load all the accent names and speaker names */
    print("Loading data")
    model = analysis.analyse_table(filename)

    #print("Analysing all")
    #main(model, outputFilepath, "{}-{}".format(outputType, "all"), do_absolute_similarity, dodist, do_ACCDIST_similarity, doMeanModel, docorrelationtables, doPhonemes, doNearestNeighbour, doquiet, minSegmentCount)

    print("Analysing only speakers with colinguals")
    new_model = get_accents_with_over_n_speakers(0, minSegmentCount, model)
    main(new_model, outputFilepath, "{}-{}".format(outputType, "-colinguals"), do_absolute_similarity, dodist, do_ACCDIST_similarity, doMeanModel, docorrelationtables, doPhonemes, doNearestNeighbour, doquiet, minSegmentCount)

    #for t in new_model.transcriptions:
    #    print "Analysing model without segment %s" % t
    #    limited_model = get_transcriptions_except_t(t, new_model)
    #    main(limited_model, "without-%s" % t, do_absolute_similarity, dodist, do_ACCDIST_similarity, doMeanModel, docorrelationtables, doPhonemes, doNearestNeighbour, doquiet)

if __name__ == "__main__":
    opts, args = getopt.getopt(sys.argv[1:], "IacqmDdLf")
    callACCDIST(opts)